//#define VER_MONO

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using PAGfx.Common;

/*
 \\/|//\//   ||
 | |_\ |/_||/| |  \  /    | PAGfx Frontend v0.9     |
 |  __/ _  |\_//  / /  \    | By: Team PAlib          |
 |_/ |_/ |_|___/|_| /_/\_\   | Last Modified: 21.07.09 |
                   Frontend   |_________________________|
*/

namespace PAGfx_Frontend {
	public partial class Form1 : Form {
		public Form1() {
			InitializeComponent();
		}

		void warning(string t) {
			lastMsg.Text = t;
			System.Media.SystemSounds.Exclamation.Play();
		}

		void usrmsg(string t) {
			lastMsg.Text = t;
		}

		void infomsg(string t) {
			lastMsg.Text = t;
			System.Media.SystemSounds.Asterisk.Play();
		}
		
		string relPath = "";

		#region Toolbar Code

		//Button 1: Add item
		private void toolStripButton1_Click(object sender, EventArgs e) {
			openFileDialog1.ShowDialog();
		}

		//Button 2: Remove item
		private void toolStripButton2_Click(object sender, EventArgs e) {
			if(Get_Current_Grid().SelectedRows.Count == 1)
				Get_Current_Grid().Rows.Remove(Get_Current_Grid().SelectedRows[0]);
			//else
			//    Beep;
		}

		//Button 3 : Load INI
		private void toolStripButton3_Click(object sender, EventArgs e) {
			Grid_LoadINI();
		}

		//Button 4: Save INI
		private void toolStripButton4_Click(object sender, EventArgs e) {
			Grid_SaveINI();
		}

		//Button 5 : Save and Convert
		private void toolStripButton5_Click(object sender, EventArgs e) {
			Grid_SaveINIConvert();
		}

		//Button 6: About
		private void toolStripButton6_Click(object sender, EventArgs e) {
			Form frmAbout = new FormAbout();
			frmAbout.ShowDialog();
		}

		//Button 7: Update Row
		private void toolStripButton7_Click(object sender, EventArgs e) {
			if(Get_Current_Grid().SelectedRows.Count == 1)
				Grid_SaveINIConvertRow();
			//else
			//    Beep;
		}

		#endregion

		#region Grid routines (Grid_)

		// Add items
		private void Grid_AddFile(string File) {
			// Todo: Add some sort of extension check here!

			string[] SplitPath = File.Split(new char[] { '\\' });
			string ShortName = SplitPath[SplitPath.GetLength(0) - 1];

			switch(TabControl1.SelectedTab.Text) {
				case "Sprites":
					Grid_Sprite.Rows.Add(ShortName, "256colors", LoadINI.ExtractName(ShortName), File);
					break;
				case "Backgrounds":
					Grid_BG.Rows.Add(ShortName, "EasyBg", LoadINI.ExtractName(ShortName), File);
					break;
				case "Textures":
					Grid_Tex.Rows.Add(ShortName, "256colors", LoadINI.ExtractName(ShortName), File);
					break;
			}
			DataGridView Grid = Get_Current_Grid();
			Grid.Rows[Grid.Rows.Count - 1].Selected = true;
			UpdatePreview(Grid);
			CheckImageDepth(Grid);
		}

		// Global grid change event
		private void Grid_CellValueChanged(object Grid, DataGridViewCellEventArgs e) {
			CheckImageDepth((DataGridView) Grid);
		}

		private void CheckImageDepth(DataGridView Grid) {
			if(Grid.Rows.Count == 0)
				return;			
			usrmsg("Ready.");
			
			// TODO: Implement this on Mono
#if (!VER_MONO)
			switch(Grid.SelectedRows[0].Cells[1].Value.ToString()) {
				case "16colors":
					if(pictureBox1.Image.PixelFormat == System.Drawing.Imaging.PixelFormat.Format4bppIndexed)
						warning("WARNING: Not a 16 colors image!");
					break;
				case "16bit":
					if(pictureBox1.Image.Palette.Entries.Length > 0)
						warning("WARNING: Not a true color image!");
					break;
				default:
					if(pictureBox1.Image.PixelFormat != System.Drawing.Imaging.PixelFormat.Format8bppIndexed)
						warning("WARNING: Not a 256 colors image!");
					break;
			}
#endif
		}

		// Global grid click event
		private void Grid_RowChanged(object Grid, EventArgs e) {
			UpdatePreview((DataGridView) Grid);
		}

		private void UpdatePreview(DataGridView Grid) {
			if(Grid.SelectedRows.Count == 0) {
				pictureBox1.Image = null;
				lblPInfo.Text = "";
				return;
			}
			try {
				pictureBox1.Image = new Bitmap(Grid.SelectedRows[0].Cells[3].Value.ToString());
				lblPInfo.Text = "Info: " + pictureBox1.Image.Width.ToString() + "x" + pictureBox1.Image.Height.ToString();
				if(pictureBox1.Image.Palette.Entries.Length > 0)
					lblPInfo.Text += " - " + pictureBox1.Image.Palette.Entries.Length.ToString() + " colors";
				else
					lblPInfo.Text += " - full color";
			} catch(Exception ex) {
				MessageBox.Show("Error: " + ex.Message);
			}
		}

		//Get the current grid
		private DataGridView Get_Current_Grid() {
			switch(TabControl1.SelectedTab.Text) {
				case "Sprites":
					return Grid_Sprite;
				case "Backgrounds":
					return Grid_BG;
				case "Textures":
					return Grid_Tex;
			}
			return null;
		}

		//Get content for export
		private string Grid_GetContent(DataGridView Grid, bool All) {
			string Output = "";
			if(All)
				foreach(DataGridViewRow SelRow in Grid.Rows) {
					// Add filename
					string filename = SelRow.Cells[3].Value.ToString();
					string colormode = SelRow.Cells[1].EditedFormattedValue.ToString();
					string palette = (SelRow.Cells[2].Value != null) ? SelRow.Cells[2].Value.ToString() : "";

					if(filename.Length > relPath.Length)
						if((filename.Substring(0, relPath.Length) == relPath) ||
					   		(filename.Substring(0, relPath.Length) == relPath))
							filename = filename.Substring(relPath.Length);

					// Final output
					Output = Output + filename + " " + colormode + " " + palette + "\r\n";
				} else {
				DataGridViewRow SelRow = Grid.SelectedRows[0];
				if(SelRow != null) {
					Output = SelRow.Cells[3].Value.ToString() + " " + SelRow.Cells[1].Value.ToString() + " ";
					Output += (SelRow.Cells[2].Value != null) ? SelRow.Cells[2].Value.ToString() : "None" + "\r\n";
				}
			}
			return Output;
		}

		#endregion

		#region Drag and Drop Code

		private void Grid_DragCheck(object sender, DragEventArgs e) {
			if(e.Data.GetDataPresent(DataFormats.FileDrop))
				e.Effect = DragDropEffects.Copy;
			else
				e.Effect = DragDropEffects.None;
		}

		private void Grid_DragDrop(object sender, DragEventArgs e) {
			if(e.Data.GetDataPresent(DataFormats.FileDrop)) {
				string[] files = (string[]) e.Data.GetData(DataFormats.FileDrop);
				foreach(string file in files)
					Grid_AddFile(file);
			}
		}

		#endregion

		#region Save INI

		public string SaveINI(string IniName, bool All) {
			string Output_String = "";
			// Damn Windows and it's non-standardness
			relPath = Directory.GetCurrentDirectory();
#if (!VER_MONO)
			relPath += "\\";
#else
			relPath += "/";
#endif

			string ColorS = CmdTransCol.BackColor.ToString();
			ColorS = ColorS.Substring(7, ColorS.Length - 8);
			if(ColorS == "Lime") ColorS = "Green";
			//if(ColorS.Substring(0, 2) == "A=") {
			if((ColorS != "Black") && (ColorS != "White") && (ColorS != "Magenta") && (ColorS != "Green")){
				// User-defined color
				// Let's convert it
				string r = String.Format("{0,2:X}", CmdTransCol.BackColor.R).Replace(' ', '0');
				string g = String.Format("{0,2:X}", CmdTransCol.BackColor.G).Replace(' ', '0');
				string b = String.Format("{0,2:X}", CmdTransCol.BackColor.B).Replace(' ', '0');
				ColorS = r + g + b;
			}

			Output_String = "#TranspColor " + ColorS + "\r\n";

			//Sprites
			if(All || TabControl1.SelectedTab.Text == "Sprites")
				Output_String = Output_String + "\r\n" + // Had to put \r\n as line break
							"#Sprites :\r\n" +
							Grid_GetContent(Grid_Sprite, All);

			//Backgrounds
			if(All || TabControl1.SelectedTab.Text == "Backgrounds")
				Output_String = Output_String + "\r\n" +
							"#Backgrounds :\r\n" +
							Grid_GetContent(Grid_BG, All);

			//Textures
			if(All || TabControl1.SelectedTab.Text == "Textures")
				Output_String = Output_String + "\r\n" +
							"#Textures :\r\n" +
							Grid_GetContent(Grid_Tex, All);

			//Save
			FileStream IniFile = null;
			BinaryWriter IniWriter = null;
			try {
				IniFile = new FileStream(System.AppDomain.CurrentDomain.BaseDirectory + IniName, FileMode.Create);
				IniWriter = new BinaryWriter(IniFile);
				IniWriter.Write(Encoding.ASCII.GetBytes(Output_String));
			} catch(Exception ex) {
				MessageBox.Show("Error: " + ex.Message);
			} finally {
				if(IniWriter != null) IniWriter.Close();
				if(IniFile != null) IniFile.Close();
			}

			return Output_String;
		}

		public void Grid_SaveINI() {
			string output = SaveINI("PAGfx.ini", true);
			//MessageBox.Show(output, "Saved");
			infomsg("INI successfully exported.");
		}

		public void Grid_SaveINIConvert() {
			SaveINI("PAGfx.ini", true);
			System.Diagnostics.Process.Start("PAGfx.exe");
		}

		public void Grid_SaveINIConvertRow() {
			SaveINI("PAGfx_tmp.ini", false);
			System.Diagnostics.Process.Start("PAGfx.exe", "PAGfx_tmp.ini");
		}

		#endregion

		#region Load INI file

		public void Grid_LoadSprites() {
			// Load the sprites
			Grid_Sprite.Rows.Clear();
			for(int i = 0; i < LoadINI.NSprites; i++) {
				if(string.IsNullOrEmpty(LoadINI.IniSprites[i].PaletteName))
					LoadINI.IniSprites[i].PaletteName = "None";
				if(File.Exists(LoadINI.IniSprites[i].Path))
					Grid_Sprite.Rows.Add(LoadINI.IniSprites[i].Name, LoadINI.SpriteModes[LoadINI.IniSprites[i].ColorMode], LoadINI.IniSprites[i].PaletteName, LoadINI.IniSprites[i].Path);
				else
					MessageBox.Show("File " + LoadINI.IniSprites[i].Path + " not found");
			}
		}

		public void Grid_LoadTextures() {
			// Load the sprites
			Grid_Tex.Rows.Clear();
			for(int i = 0; i < LoadINI.NTextures; i++) {
				if(string.IsNullOrEmpty(LoadINI.IniTextures[i].PaletteName))
					LoadINI.IniTextures[i].PaletteName = "None";
				if(File.Exists(LoadINI.IniTextures[i].Path))
					Grid_Tex.Rows.Add(LoadINI.IniTextures[i].Name, LoadINI.TextureModes[LoadINI.IniTextures[i].ColorMode], LoadINI.IniTextures[i].PaletteName, LoadINI.IniTextures[i].Path);
				else
					MessageBox.Show("File " + LoadINI.IniTextures[i].Path + " not found");
			}
		}

		// Added to block invalid background modes
		public string Grid_BgModeChk(string bgmode){
			switch(bgmode){
				case "EasyBg":
				case "UnlimitedBg":
				case "RotBg":
				case "8bit":
				case "16bit":
				case "16cFont":
				case "8bitFont":
				case "1bitFont":
					return bgmode;
				default:
					switch(bgmode){
						case "TileBg":
						case "LargeMap":
							MessageBox.Show(bgmode + " is obsolete -> using EasyBg");
							break;
						default:
							MessageBox.Show(bgmode + " is invalid -> using EasyBg");
							break;
					}
					return "EasyBg";
			}
		}

		public void Grid_LoadBgs() {
			// Load the backgrounds
			Grid_BG.Rows.Clear();
			for(int i = 0; i < LoadINI.NBackgrounds; i++) {
				if(string.IsNullOrEmpty(LoadINI.IniBgs[i].PaletteName))
					LoadINI.IniBgs[i].PaletteName = "None";
				if(File.Exists(LoadINI.IniBgs[i].Path))
					Grid_BG.Rows.Add(LoadINI.IniBgs[i].Name, Grid_BgModeChk(LoadINI.BgModes[LoadINI.IniBgs[i].BgMode]), LoadINI.IniBgs[i].PaletteName, LoadINI.IniBgs[i].Path);
				else
					MessageBox.Show("File " + LoadINI.IniBgs[i].Path + " not found");
			}
		}

		public void Grid_LoadINI() {
			// Parses the ini file !
			string loaded = LoadINI.IniParse("PAGfx.ini");
			if(loaded == "")
				loaded = string.Format("Loaded:\n{0} sprite{1}\n{2} background{3}\n{4} texture{5}",
					LoadINI.NSprites, LoadINI.NSprites == 1 ? "" : "s",
					LoadINI.NBackgrounds, LoadINI.NBackgrounds == 1 ? "" : "s",
					LoadINI.NTextures, LoadINI.NTextures == 1 ? "" : "s");

			Grid_LoadTranspColor();
			Grid_LoadSprites();
			Grid_LoadBgs();
			Grid_LoadTextures();

			UpdatePreview(Get_Current_Grid());

			infomsg(loaded);
		}

		void Grid_LoadTranspColor() {
			// Loads the correct transparent color...
			UInt16 clr = LoadINI.TranspColor.Color;
			switch(LoadINI.TranspColor.Name) {
				case "White":
					CmdTransCol.BackColor = System.Drawing.Color.White; break;
				case "Black":
					CmdTransCol.BackColor = System.Drawing.Color.Black; break;
				case "Green":
					CmdTransCol.BackColor = System.Drawing.Color.Lime; break;
				case "Magenta":
					CmdTransCol.BackColor = System.Drawing.Color.Magenta; break;
				default:
					CmdTransCol.BackColor = System.Drawing.Color.FromArgb(
					    (int)(clr & 0x1F) << 3, (int)((clr >> 5) & 0x1F) << 3, (int)((clr >> 10) & 0x1F) << 3);
					break;
			}
		}

		#endregion

		#region Dialogs

		// Transparant color button
		private void CmdTransCol_Click(object sender, EventArgs e) {
			ColorMenu.Show(CmdTransCol, new Point(10, 20));
		}

		// Transparent color contextmenu...
		private void ColorMenu_ItemClicked(object sender, ToolStripItemClickedEventArgs e) {
			switch(e.ClickedItem.Text) {
				case "White":
					CmdTransCol.BackColor = System.Drawing.Color.White;
					break;
				case "Black":
					CmdTransCol.BackColor = System.Drawing.Color.Black;
					break;
				case "Green":
					CmdTransCol.BackColor = System.Drawing.Color.Lime;
					break;
				case "Magenta":
					CmdTransCol.BackColor = System.Drawing.Color.Magenta;
					break;
				case "Custom...":
					ColorMenu.Hide();
					Color tmp = colorDialog1.Color;
					if(colorDialog1.ShowDialog() == DialogResult.Cancel) {
						colorDialog1.Color = tmp;
						break;
					}
					CmdTransCol.BackColor = colorDialog1.Color;
					break;
			}
		}

		bool isInvalidName(string n) {
			if((n[0] >= '0') && (n[0] <= '9'))
				return true;
			foreach(char c in n)
				if("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_".IndexOf(c) == -1)
					return true;
			return false;
		}

		//Open File dialog
		public void openFileDialog1_FileOk(object sender, CancelEventArgs e) {
			String[] files = openFileDialog1.FileNames;
			foreach(string file in files) {
				string filenopath = Path.GetFileNameWithoutExtension(file);
				if(isInvalidName(filenopath)) {
					MessageBox.Show("The filename \"" + filenopath + "\" is an invalid C identifier!\r\n" +
						"Invalid characters will be replaced by an underscore when converting (\"_\").");
				}
				Grid_AddFile(file);
			}
		}

		private void TabControl1_SelectedIndexChanged(object sender, EventArgs e) {
			UpdatePreview(Get_Current_Grid());
		}

		private void TabControl1_Click(object sender, EventArgs e) {
			usrmsg("Ready.");
		}

		#endregion
	}
}
